import {
  Action,
  getModule,
  Module,
  Mutation,
  MutationAction,
  VuexModule
} from 'vuex-module-decorators';
import findIndex from 'lodash.findindex';
import { Measure, MeasureConfig, Measurement, MeasurementMeta } from '@/models';
import assign from 'lodash.assign';
import api from '@/services/pm-api-client';
import { logService } from '@/services/logging';

export interface MeasurementStore { [key: string]: Array<Measurement<MeasurementMeta>>; }
export interface SetMeasurementsParameters {
  measure: Measure<MeasureConfig>;
  measurements: Array<Measurement<MeasurementMeta>>;
}

const logger = logService.getLogger('/store-modules/measurement');

@Module({ namespaced: true, name: 'measurement' })
export class MeasurementStoreModule extends VuexModule {
  public measurements: MeasurementStore = {};

  @Action({ rawError: true })
  public async fetchMeasurements(measure: Measure<MeasureConfig>) {
    logger.trace('Fetching measurements for measure ' + measure.id);
    const measurements = await api.getMeasurements(measure.slug); // assumption: always returns at least []
    this.context.commit('SET_MEASUREMENTS', { measure, measurements });
  }

  @Mutation
  public SET_MEASUREMENTS({ measure: measure, measurements: measurements }: SetMeasurementsParameters) {
    this.measurements = assign({}, this.measurements, { [measure.id]: measurements });
  }

}
